#include <cstring>
#include <cstdlib>
#include <cstdio>

#include "myspell.hxx"

/*
 *  MIspell -- (Magyar) MySpell with Ispell pipe interface
 *
 */

#define LIBDIR "/usr/lib/myspell/"

extern char * mystrdup(const char * s);

using namespace std;

struct wordlist {
  char * word;
  wordlist * next;
};

void log(char * message)
{
	FILE *f = fopen("/tmp/mispell.log","a");
	fprintf(f,"%s\n",message);
	fclose(f);
}


void putdic(char * word, MySpell * pMS)
{
	char * w;

	pMS->spell("PuT");
//	if ((strlen(word)>4) && (w=strstr(word+1,"PuT"))!=NULL) {
	if ((w=strstr(word+1,"PuT"))!=NULL) {
		*w='\0';
		pMS->spell(word);
		pMS->spell(w+3);				
	} else {
		pMS->spell(word);
		pMS->spell(word);
	}
}

void load_privdic(char * filename, MySpell * pMS) 
{
	char buf[8192];
	
	///fprintf(stdout,"%s\n",filename);
	FILE *dic = fopen(filename,"r");
	if (dic) {
		while(fgets(buf,8191,dic)) {
			*(buf + strlen(buf) - 1) = '\0';
	///fprintf(stdout,"%s\n",buf);
			putdic(buf,pMS);
		}
	}
}

int exist(char * filename)
{
	FILE *f = fopen(filename,"r");
	if (f) {
		fclose(f);
		return 1;
	}
	return 0;
}

int save_privdic(char * filename, char * filename2, wordlist * w)
{
	wordlist * r;
	FILE *dic = fopen(filename,"r");
	if (dic) {
		fclose(dic);
		dic = fopen(filename,"a");
	} else {
		dic = fopen(filename2,"a");	
	}
	if (! dic) return 0;
	while (w != NULL) {
		fprintf(dic,"%s\n",w->word);
		log("DIC:");log(w->word);log("\n");
		r = w;
		free(w->word);
		w = w->next;
		free(r);

	}
	fclose(dic);
	return 1;
}

int main(int argc, char** argv)
{
	MySpell * pMS = NULL;
//	char * faff;
//	char * fdic;
//	char * wtc;
//	char *privdicname;
//	FILE* privdic;

int TeXMODE=0;
char buf[8192];
wordlist * dicwords = NULL;

int argstate=0;
char * dicname=NULL;

for(int i=1; i<argc; i++) {
	log(argv[i]);

	if ((argstate==1) && (! dicname)) {
		dicname=mystrdup(argv[i]);
		argstate=0;
	}
	if (strcmp(argv[i],"-d")==0) argstate=1;
	if (strcmp(argv[i],"-h")==0) {
		fprintf(stderr,"MIspell -- MySpell Ispell pipe interface\n");
		fprintf(stderr,"Author: Lszl Nmeth, 2002, License: GNU GPL\n");
		fprintf(stderr,"Usage: mispell [-d dicname]\n");
		exit(0);
	}
	if (strcmp(argv[i],"-vv")==0) {
		fprintf(stdout,"@(#) MIspell with Ispell Pipe interface Version 3.1.20 07/04/02 (0.5)\n");
		exit(0);
	}
}

if (dicname==NULL) dicname = "default";

strcpy(buf,LIBDIR);
char * faff = buf+strlen(buf);
strcat(buf,dicname);
strcat(buf,".aff");
char * buf2 = buf+strlen(buf)+1;
strcpy(buf2,LIBDIR);
char * fdic = buf2+strlen(buf2);
strcat(buf2,dicname);
strcat(buf2,".dic");

if (exist(faff) && exist(fdic)) {
	pMS = new MySpell(faff,fdic);
}

if ((!pMS) && exist(buf) && exist(buf2)) {
	pMS = new MySpell(buf,buf2);
}

if (!pMS) {
	fprintf(stderr,"Couldn't open affix and/or dictionary files.\n");
	exit(0); /// XXX
}

    /* open the private dictionaries */
    strcpy(buf,getenv("HOME"));
    strcat(buf,"/.ispell_");
    strcat(buf,dicname);    
    load_privdic(buf, pMS);
    strcpy(buf,".ispell_");
    strcat(buf,dicname);
    load_privdic(buf, pMS);

    fprintf(stdout,"@(#) MySpell with Ispell Pipe Interface Version 3.2.0, 06/17/02\n");
    fflush(stdout);
    int k;
    int dp;
    char * token;
    int pos;
    char prevnochar='\0';

    char * try_string = pMS->get_try_string();
    int try_string_len = strlen(try_string);

//    fprintf(stdout,"%s\n",pMS->get_try_string());
    while(fgets(buf,8191,stdin)) {
      k = strlen(buf);
      
      log("Q"); log(buf);
      
      *(buf + k - 1) = '\0';      
      if (TESTAFF("*@#~+-!%",buf[0],8)) { // `^' handled later 
        switch (buf[0]) {
    	    case '+': { TeXMODE=1; break; }
    	    case '-': { TeXMODE=0; break; }
    	    case '@': { putdic(buf+1,pMS); break; }
    	    case '*': {
		struct wordlist* i = 
			(struct wordlist *) malloc (sizeof(struct wordlist));
		i->word = mystrdup(buf+1);
		i->next = dicwords;
		dicwords = i;
		log(buf+1);
		putdic(buf+1,pMS);
	    }
	    case '#': {
		strcpy(buf,getenv("HOME"));
		strcat(buf,"/.ispell_magyar");
		if (save_privdic(".ispell_magyar", buf, dicwords)) dicwords=NULL;
		break;
	    }
	}
	continue;      
      }
      if (buf[0] == '^') { pos = 1; // ^ ispell command char
		// XXX Emacs ispell.el has a hidden mistake:
		// after the replacings often checks a letter
		// we ignore these checks
          if (k==3) {
		fprintf(stdout,"\n");
		continue; 
		}		
      }
      else pos = 0;
      token = buf+pos;
      for (int i=pos; i<k; i++) {
        if (! TESTAFF(try_string, buf[i], try_string_len)) {
	  if ((buf+i-token==0) || // nonchar sequence
	      ((TeXMODE==1) && (prevnochar=='\\'))) {
	    prevnochar = buf[i];
	    token = buf+i+1;
	    continue;
	  }
	  prevnochar = buf[i];
	  buf[i] = '\0';
          dp = pMS->spell(token);
          if (dp) {
            fprintf(stdout,"*\n");
	    fflush(stdout);
          } else {
            char ** wlst;
            int ns = pMS->suggest(&wlst,token);
	    if (ns==0) {
	      fprintf(stdout,"# %s %d",token,token-buf);
	    } else {
	      fprintf(stdout,"& %s %d %d: %s",token,ns,token-buf,wlst[0]);
	    }
            for (int i=1; i < ns; i++) {
              fprintf(stdout,", %s",wlst[i]);
              free(wlst[i]);
            }
            fprintf(stdout,"\n");
	    fflush(stdout);
            free(wlst);
          }
	  token = buf+i+1;
	}
      }
      fprintf(stdout,"\n");
      fflush(stdout);
    }
    if (try_string) free(try_string);
    delete pMS;
    return 0;
}

